import pygame
import random
from pygame.locals import *

from mediator import mediator

import player
import events

CONTAINER_COLOR = (90,90,90)
BACKGROUND_COLOR = (0,0,0)
HEALTHY_COLOR = (10,255,10)
AVERAGE_COLOR = (255,255,10)
OMG_COLOR = (255,10,10)
TEXT_COLOR = (200,200,200)

FAITH_POOL_POS0 = pygame.Rect(10, 200, 100, 200)
FAITH_POOL_POS1 = pygame.Rect(690, 200, 100, 200)

class FaithPool(pygame.sprite.Sprite):
    faith_points = 0
    max_points = 0
    
    def __init__(self, points, position, player=None):
        """Initialize a faith pool"""
        self.faith_points = points
        self.max_points = points
        self.rect = position
        self.player = player
        self._image = None
        mediator.addListener('turn_end', self.resetPoolOnEndOfAttack)
    
    def faith_points():
        doc = "faith points value displayed"
        def fget(self):
            return self._faith_points
        def fset(self, value):
            self._faith_points = value
            self._image = None
        return locals()
    faith_points = property(**faith_points())
    
    def increase_max(self, arg):
        self.max_points += arg
        
    def decrease_max(self, arg):
        self.max_points -= arg
            
    def increase_pool(self, arg):
        self.faith_points += arg
        
    def decrease_pool(self, arg):
        self.faith_points -= arg
    
    def reset(self):
        self.faith_points = self.max_points

    def resetPoolOnEndOfAttack(self, event):
        # todo: find out faith left and calculate it here
        if(event.turnType=='attack' and event.player==self.player):
            #Adjust both player's pools
            for fp in faithPools:
                fp.max_points = fp.player.health
                fp.player.faith = fp.player.health
                fp.reset()
        pass

    
    def image():
        doc = "The image property."
        def fget(self):
            if (not self._image):
                self._image = self.render()
            return self._image
        return locals()
    image = property(**image()) 

    def remaining():
        doc = "The remaining property."
        def fget(self):
            return self.faith_points
        def fset(self, value):
            self.faith_points = value
        return locals()
    remaining = property(**remaining())
    
    def value_color():
        doc = "The value_color property."
        def fget(self):
            if(self.percentage > 0.6):
                return HEALTHY_COLOR
            elif(self.percentage > 0.3):
                return AVERAGE_COLOR
            return OMG_COLOR
        return locals()
    value_color = property(**value_color())
    
    def percentage():
        doc = "The percentage property."
        def fget(self):
            try:
                return self.faith_points / float(self.max_points)
            except ZeroDivisionError:
                return 0
        def fset(self, value):
            self.faith_points = value * self.max_points
        return locals()
    percentage = property(**percentage())

    def render(self):
        #Make the math easier to understand
        width = self.rect.width
        height = self.rect.height
        margined_width = self.rect.width * 0.3
        margined_height = self.rect.height * 0.9
        height_margin = (height - margined_height) / 2.0
        width_margin = (width - margined_width) / 2.0
        value_height = int(self.percentage * margined_height)
        value_height_padding = height_margin + (margined_height - value_height)
        
        s = pygame.Surface((width, height)).convert_alpha()
        s.fill((0,0,0,0))
        
        container_points = [
            (width_margin, height_margin),
            (margined_width + width_margin, height_margin),
            (margined_width + width_margin, margined_height+height_margin),
            (width_margin, margined_height+height_margin)
            ]
        bar_points = [
            (width_margin, value_height_padding),
            (margined_width + width_margin, value_height_padding),
            (margined_width + width_margin, margined_height+height_margin),
            (width_margin, margined_height+height_margin)
            ]
        pygame.draw.polygon(s, CONTAINER_COLOR, container_points)
        pygame.draw.polygon(s, self.value_color, bar_points)
        
        if(not pygame.font.get_init()):
            pygame.font.init()
        f = pygame.font.Font(None, height / 10)
        max_dim = f.size(str(self.max_points))
        value_dim = f.size(str(self.faith_points))
        
        s.blit(f.render(str(self.max_points), True, TEXT_COLOR), (margined_width + width_margin + 2, height_margin))
        s.blit(f.render(str(self.faith_points), True, TEXT_COLOR), (margined_width - value_dim[0] - 2, value_height_padding))
        return s

faithPool0 = FaithPool(player.INITIAL_FAITH_POINTS, FAITH_POOL_POS0,
                       player=player.player0)
faithPool1 = FaithPool(player.INITIAL_FAITH_POINTS, FAITH_POOL_POS1,
                      player=player.player1)
faithPools = [faithPool0, faithPool1]

def resetFaithPools():
    faithPool0.faith_points = player.INITIAL_FAITH_POINTS
    faithPool1.faith_points = player.INITIAL_FAITH_POINTS
    faithPool0.max_points = player.INITIAL_FAITH_POINTS
    faithPool1.max_points = player.INITIAL_FAITH_POINTS

def updatePoolOnPointsChangeHandler(event):
    for pool in (faithPool0, faithPool1):
        if (event.player is pool.player):
            pool.faith_points = event.player.faith
mediator.addListener(
    events.PLAYER_FAITH_ADJUSTED, updatePoolOnPointsChangeHandler)



# Some test code to animate a faith pool
if __name__=='__main__':
    screen = pygame.display.set_mode([800, 600])
    rect = pygame.Rect((20,20), (200,200))
    fp = FaithPool(1000,rect)
    while ((pygame.event.poll().type != KEYDOWN) and fp.remaining > 0):
        screen.blit(fp.image, fp.rect)
        fp.decrease_pool(random.randint(0,100))
        pygame.time.delay(1000)
        pygame.display.update()
