'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "lib"
directory.
'''
import pygame
from pygame.locals import *

pygame.init()

W,H = 800,600
screen = pygame.display.set_mode((W,H),SWSURFACE)

import data
import menu
import turn
import sound
import terrain
import messages
import player
import wtf
import faith_pool
import spinny
import music

from pgu import gui
from mediator import mediator

# Listeners for menus...
def quitListener(event):
    global _quit
    _quit = 1

def startListener(event):
    print "Start button was pressed!"
    menu.start_menu.undisplayMenu()
    mediator.publishEvent("start_turns")
    menu.in_game_menu.displayMenu()

def wtfListener(event):
    print "wtf button was pressed!"
    wtf.active = True
    
def gameOverListener(event):
    global g_terrain
    
    print "The game is over!"
    # reset the menus!
    menu.undisplayAllMenus()
    # reset the terrain
    g_terrain = terrain.Terrain((W, H))
    # reset the faith points
    player.resetFaithPoints()
    # reset the pool points
    faith_pool.resetFaithPools()
    # reset the villiage
    
    menu.start_menu.displayMenu()
    

def main():
    global _quit, g_terrain
    
    tf_screen0 = pygame.Surface((W,H)).convert_alpha()
    tf_screen0.fill((0,0,0,0))
    pygame.draw.polygon(tf_screen0, (0,0,0,64), [(0,0),(W,H),(W,0)])
    tf_screen1 = pygame.Surface((W,H)).convert_alpha()
    tf_screen1.fill((0,0,0,0))
    pygame.draw.polygon(tf_screen1, (0,0,0,64), [(0,0),(W,H),(0,H)])
    
    # Init sound
    if (pygame.mixer):
        pygame.mixer.init(44100)

    # Our initialization code goes here.  Keep it to one
    # call per module/thing if you can. (JS)
    app, form = menu.initialize()
    mediator.addListener("quit_game", quitListener)
    mediator.addListener("start_game", startListener)
    mediator.addListener("wtf", wtfListener)
    mediator.addListener("game_over", gameOverListener)
    spinny.setup(screen)
    # Game init event
    mediator.publishEvent('init_game')

    g_terrain = terrain.Terrain((W, H))
    
    # Main game event loop.  Everything that needs to be done 
    # on a regular basis, or needs to catch events, goes in
    # this loop. (JS)
    _form = form.results()
    _quit = 0
    while not _quit:
        _form = form.results()
        screen.fill((0,0,0))
        for e in pygame.event.get():
            if ((e.type is QUIT)
                or (e.type is KEYDOWN and e.key == K_ESCAPE)):
                mediator.publishEvent("quit_game")
            if ((e.type is KEYDOWN and wtf.active)):
                wtf.active = False
            elif e.type == pygame.locals.MOUSEBUTTONDOWN and e.button in [1,3]:
                mediator.publishEvent(
                    "mouse_button_down", pos=pygame.mouse.get_pos(),
                    button=e.button)
            elif e.type == pygame.locals.MOUSEBUTTONUP and e.button in [1,3]:
                mediator.publishEvent(
                    "mouse_button_up", pos=pygame.mouse.get_pos(),
                    button=e.button)
            # Pass the event off to PGU
            app.event(e)

        # Generate mouse position events.
        mediator.publishEvent("mouse_pos", pos=pygame.mouse.get_pos())
        
        # Draw everything.
        screen.blit(g_terrain.render(), (0,0))
        for sprite in (player.player0.huts + player.player1.huts
                + turn.twisterz + faith_pool.faithPools 
                + [player.player0] + [player.player1]):
            screen.blit(sprite.image, sprite.rect)
        if turn.in_terraform:
            if turn.current_turn.player == player.player0:
                screen.blit(tf_screen0, (0,0))
            else:
                screen.blit(tf_screen1, (0,0))
        if messages.message:
            screen.blit(messages.message.image, messages.message.rect)
        if wtf.active:
            w = wtf.WhatTheFrick(pygame.Rect(0,0,600,600))
            screen.blit(w.help_surface(), (100,0))

        spinny_state_obj = spinny.spinny_state_obj
        if (spinny_state_obj.active):
            spinny.spinny_state_obj.paint()
            if turn.current_turn.player == player.player0:
                screen.blit(tf_screen0, (0,0))
            else:
                screen.blit(tf_screen1, (0,0))
                
        app.paint(screen)
        pygame.display.flip()
        pygame.time.wait(10)
        mediator.publishEvent('tick')

    # Wait for music to fade out
    for x in xrange(270):
        pygame.time.wait(10)


