import sys
import os

import pygame
from pygame.locals import *
from pgu import text, gui as pgui

import spinny_state

import menu
from mediator import mediator
from mediator import Event
import twister
import main
import turn
import player
import twister_info
import spinny_arrow
import math
import random

# global
spinny_state_obj = None
twisters_player_1 = []
twisters_player_2 = []

# Range of speed a twister can have (affects how fast it moves across the screen)
MIN_SPEED = 5 
MAX_SPEED = 10

# Range of twist a twister can have (affects how much damage it does to huts
# and how much it is affected by terrain)
MIN_TWIST = -15
MAX_TWIST = 15

def undisplayAllMenus():
    menu.in_game_menu.undisplayMenu()
    menu.plan_attack_menu.undisplayMenu()
    menu.plan_attack_menu_done.undisplayMenu()
    menu.plan_attack_move_resize_menu.undisplayMenu()

    
def planAttackStartListener(event):
    global spinny_state_obj, twisters_player_1, twisters_player_2
    if (turn.current_turn == None or event.turnType != 'plan_attack'):
        return

    if (event.turnType == 'plan_attack'):
        print "Plan attack stage beginning"

        spinny_state_obj.active = True
        spinny_state_obj.setState("ADD_1")
        # erase all data
        spinny_state_obj.allsprites = pygame.sprite.RenderPlain()
        if turn.current_turn.player == player.player0:
            twisters_player_1 = []
        else:
            twisters_player_2 = []
            
        if (turn.current_turn.player.getMaxTornadoPoints() > 0):
            undisplayAllMenus()
            menu.plan_attack_menu.displayMenu()
        else:
            undisplayAllMenus()
            menu.plan_attack_menu_done.displayMenu()

def planAttackEndListener(event):
    global spinny_state_obj

    if (not spinny_state_obj.active):
        return 

    spinny_state_obj.active = False
    undisplayAllMenus()
    menu.in_game_menu.displayMenu()  

def planAttackAddTornado(event):
    global spinny_state_obj
    
    if (not spinny_state_obj.active):
        return 

    print "Plan attack - add tornado"
    undisplayAllMenus()
    menu.plan_attack_move_resize_menu.displayMenu()
    spinny_state_obj.addTornado()

def planAttackMouseButtonDown(event):
    global spinny_state_obj
    
    if (not spinny_state_obj.active):
        return 

    print "Plan attack - mouse down"
    spinny_state_obj.handleMouseDown(event)

def planAttackMouseButtonUp(event):
    global spinny_state_obj
    
    if (not spinny_state_obj.active):
        return 

    print "Plan attack - mouse up"
    spinny_state_obj.handleMouseUp(event)

def planAttackMouseMove(event):
    global spinny_state_obj

    if (not spinny_state_obj.active):
        return

    spinny_state_obj.handleMouseMove(event)

def planAttackTick(event):
    global spinny_state_obj

    if (not spinny_state_obj.active):
        return

    spinny_state_obj.handleTick()

def planAttackMoveResize(event):
    global spinny_state_obj

    if (not spinny_state_obj.active):
        return

    spinny_state_obj.doneMovingResizing()
    undisplayAllMenus()
    if (turn.current_turn.player.getMaxTornadoPoints() > 0):
        menu.plan_attack_menu.displayMenu()
    else:
        menu.plan_attack_menu_done.displayMenu()

def planAttackMoveCancel(event):
    global spinny_state_obj

    if (not spinny_state_obj.active):
        return

    spinny_state_obj.cancelMovingResizing()
    undisplayAllMenus()
    menu.plan_attack_menu.displayMenu()

def planAttackGameOverListener(event):
    global spinny_state_obj, twisters_player_1, twisters_player_2

    if (not spinny_state_obj.active):
        return

    spinny_state_obj.active = False
    spinny_state_obj.setState("ADD_1")
    # erase all data
    spinny_state_obj.allsprites = pygame.sprite.RenderPlain()
    twisters_player_1 = []
    twisters_player_2 = []

    # hide all menus
    undisplayAllMenus()

    
def planAttackDoneTurn(event):
    global spinny_state_obj, twisters_player_1, twisters_player_2

    if (not spinny_state_obj.active):
        return

    print "Plan attack DONE TURN"
    spinny_state_obj.active = False
        
    # create the twisters
    for sprite in spinny_state_obj.allsprites:
        print "Creating a twister for later!"

        x = random.randint(MIN_SPEED, MAX_SPEED) * math.cos(math.radians(sprite.angle+90)) 
        y = x * math.tan(math.radians(sprite.angle+90))
        twist = random.randint(MIN_TWIST, MAX_TWIST)

        twisterie = twister_info.TwisterInfo(initPos=sprite.last_pos,
                                radius=sprite.getRadius() + 10,
                                moveVector=(x,-y),
                                twist=twist,
                                terrain=main.g_terrain)

        if turn.current_turn.player == player.player0:
            twisters_player_1.append(twisterie)
        else:
            twisters_player_2.append(twisterie)
        

    # don't do menus the main function will do them!
    #undisplayAllMenus()
    #menu.in_game_menu.displayMenu()

    # this must be last
    print "Publishing next-turn spinny.py"
    mediator.publishEvent(Event("next_turn"))
    
def setup(screen):
    global spinny_state_obj, twisters_player_1, twisters_player_2
    
    mediator.addListener("turn_start", planAttackStartListener)
    #mediator.addListener("turn_end", planAttackEndListener)
    mediator.addListener("plan_attack_add_tornado", planAttackAddTornado)
    mediator.addListener("tick", planAttackTick)
    mediator.addListener("mouse_button_down", planAttackMouseButtonDown)
    mediator.addListener("mouse_button_up", planAttackMouseButtonUp)
    mediator.addListener("mouse_pos", planAttackMouseMove)
    mediator.addListener("plan_attack_move_resize", planAttackMoveResize)
    mediator.addListener("plan_attack_move_cancel", planAttackMoveCancel)
    mediator.addListener("plan_attack_done_turn", planAttackDoneTurn)
    mediator.addListener("game_over", planAttackGameOverListener)
    background = pygame.Surface(screen.get_size())
    spinny_state_obj = spinny_state.StateObj(screen, background)

