import pygame
from pygame.locals import *
from pgu import text, gui as pgui

import spinny_arrow
import turn
import player
from mediator import mediator

class StateObj:
    def __init__(self, screen, background):
        self.state = ""
        self.screen = screen
        self.background = background
        self.allsprites = pygame.sprite.RenderPlain()
        self.moving_sprite = None
        self.resize_sprite = None
        self.gui = None
        self.active = False
        self.current_sprite = None
        
    def setState(self, new_state):
        if (self.active == False):
            return
        
        # assume the left mouse button was clicked
        self.state = new_state
        if ("ADD_1" == self.state):
            # set up gui
            self.gui = pgui.App()        
            self.lo = pgui.Container(width=self.background.get_rect().width, height=self.background.get_rect().height)
            self.btn1 = pgui.Button("Add Tornado")
            self.btn1.connect(pgui.CLICK, StateObj.addTornado, self) 
            self.lo.add(self.btn1,0,self.background.get_rect().height-30)
            self.gui.init(self.lo)
        elif ("MOVE_RESIZE_1" == self.state):
            # set up gui
            self.gui = pgui.App()        
            self.lo = pgui.Container(width=self.background.get_rect().width, height=self.background.get_rect().height)
            self.btn1 = pgui.Button("Done Moving/Resizing")
            self.btn1.connect(pgui.CLICK, StateObj.doneMovingResizing, self) 
            self.lo.add(self.btn1,0,self.background.get_rect().height-30)
            self.gui.init(self.lo)

    def addTornado(self):
        if (self.active == False):
            return
        if ("ADD_1" == self.state):
            arrow = spinny_arrow.Arrow()
            self.current_sprite = arrow
            arrow.alive = 1
            self.allsprites.add(arrow)
            self.setState("MOVE_RESIZE_1")


    def doneMovingResizing(self):
        if (self.active == False):
            return
        if ("MOVE_RESIZE_1" == self.state):
            self.moving_sprite = None
            self.resize_sprite = None
            if (None != self.current_sprite):
                # throw out the event so we can adjust our points!
                mediator.publishEvent("tornado.added", radius=self.current_sprite.getRadius())
                self.current_sprite.alive = 0
                self.current_sprite = None
            self.setState("ADD_1")

    def cancelMovingResizing(self):
        if (self.active == False):
            return
        if ("MOVE_RESIZE_1" == self.state):
            self.moving_sprite = None
            self.resize_sprite = None
            # take the sprite out of our collection
            if (None != self.current_sprite):
                self.allsprites.remove(self.current_sprite)
                self.current_sprite = None
            self.setState("ADD_1")
        
    def handleTick(self):
        if (self.active == False):
            return
        if ("ADD_1" == self.state or "MOVE_RESIZE_1" == self.state):
            for sprite in self.allsprites.sprites():
                sprite.spin()

    def handleMouseMove(self, event):
        if (self.active == False):
            return
        
        if ("MOVE_RESIZE_1" == self.state):
            if (self.moving_sprite):
                # make sure we are moving within our bounds only
                x,y = event.pos
                width = self.background.get_rect().width
                height=self.background.get_rect().height

                mid = float(height) / float(width) * x
                if ((turn.current_turn.player == player.player0) and (y > mid)) or \
                       ((turn.current_turn.player == player.player1) and (y < mid)):
                    self.moving_sprite.move(event.pos)

            if (self.resize_sprite):
                # calculate the max size
                max_size = turn.current_turn.player.getMinTornadoPoints()
                self.resize_sprite.resize(pygame.mouse.get_rel(), max_size)

    def handleMouseDown(self, event):
        if (self.active == False):
            return

        if ("MOVE_RESIZE_1" == self.state):
            for sprite in self.allsprites.sprites():

                if (1 == event.button):
                    if (sprite.clicked(event.pos)):
                        self.moving_sprite = sprite
                if (3 == event.button):

                    if (sprite.clicked(event.pos)):
                        self.resize_sprite = sprite

    def handleMouseUp(self, event):
        if (self.active == False):
            return
        if ("MOVE_RESIZE_1" == self.state):
            if (3 == event.button):
                self.resize_sprite = None
            if (1 == event.button):
                self.moving_sprite = None


    def paint(self):
        if (self.active == False):
            return
        self.allsprites.update()
        self.allsprites.draw(self.screen)

